@@ -15,6 +15,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <signal.h>
+#include <dirent.h>
 
 /* ts A61007 */
 /* #include <a ssert.h> */
@@ -310,12 +311,41 @@ int burn_drive_inquire_media(struct burn_drive *d)
 		if (d->current_profile == -1 || d->current_is_cd_profile)
 			d->read_toc(d);
 
-		/* ts A70314 */
-		d->status = BURN_DISC_UNSUITABLE;
+		/* ts A70314 , B10712 */
+		if (d->status != BURN_DISC_EMPTY)
+			d->status = BURN_DISC_UNSUITABLE;
 	}
 	return 1;
 }
 
+/* ts B10730 */
+/* Send a default mode page 05 to CD and DVD-R-oids */
+int burn_drive_send_default_page_05(struct burn_drive *d, int flag)
+{
+	struct burn_write_opts *opts;
+
+	if (d->sent_default_page_05)
+		return 0;
+	if (!((d->status == BURN_DISC_APPENDABLE ||
+               d->status == BURN_DISC_BLANK) &&
+              (d->current_is_cd_profile || d->current_profile == 0x11 ||
+	       d->current_profile == 0x14 || d->current_profile == 0x15)))
+		return 0;
+	opts = burn_write_opts_new(d);
+	if (opts == NULL)
+		return -1;
+	if (d->status == BURN_DISC_APPENDABLE)
+		burn_write_opts_set_write_type(opts,
+			BURN_WRITE_TAO, BURN_BLOCK_MODE1);
+	else
+		burn_write_opts_set_write_type(opts,
+			BURN_WRITE_SAO, BURN_BLOCK_SAO);
+	d->send_write_parameters(d, opts);
+	burn_write_opts_free(opts);
+	d->sent_default_page_05 = 1;
+	return 1;
+}
+
 
 int burn_drive_grab(struct burn_drive *d, int le)
 {
@@ -379,6 +409,8 @@ int burn_drive_grab(struct burn_drive *d, int le)
 		d->silent_on_scsi_error = 1;
 	/* ts A61125 : outsourced media state inquiry aspects */
 	ret = burn_drive_inquire_media(d);
+
+	burn_drive_send_default_page_05(d, 0);
 	d->silent_on_scsi_error = sose;
 	d->busy = BURN_DRIVE_IDLE;
 	return ret;
@@ -1201,7 +1233,9 @@ int burn_drive_info_forget(struct burn_drive_info *info, int force)
 
 void burn_drive_info_free(struct burn_drive_info drive_infos[])
 {
+#ifndef Libburn_free_all_drives_on_infO
 	int i;
+#endif
 
 /* ts A60904 : ticket 62, contribution by elmom */
 /* clarifying the meaning and the identity of the victim */
@@ -2042,6 +2076,7 @@ int burn_drive_convert_fs_adr_sub(char *path, char adr[], int *rec_count)
 	return 0;
 }
 
+/* API */
 /** Try to convert a given existing filesystem address into a persistent drive
     address.  */
 int burn_drive_convert_fs_adr(char *path, char adr[])
@@ -2053,6 +2088,88 @@ int burn_drive_convert_fs_adr(char *path, char adr[])
 }
 
 
+/* API */
+int burn_lookup_device_link(char *dev_adr, char link_adr[],
+			char *dir_adr, char **ranks, int rank_count, int flag)
+{
+	DIR *dirpt= NULL;
+	struct dirent *entry;
+	struct stat link_stbuf;
+	char *adr= NULL, *namept, *sys_adr= NULL;
+	int ret, name_rank, found_rank= 0x7fffffff, dirlen, i, rec_count = 0;
+	static char default_ranks_data[][8] =
+		{"dvdrw", "cdrw", "dvd", "cdrom", "cd"};
+	char *default_ranks[5];
+
+	link_adr[0] = 0;
+	if (ranks == NULL) {
+		for (i = 0; i < 5; i++)
+			default_ranks[i] = default_ranks_data[i]; 
+		ranks = default_ranks;
+		rank_count= 5;
+        }
+	dirlen= strlen(dir_adr) + 1;
+	if (strlen(dir_adr) + 1 >= BURN_DRIVE_ADR_LEN) {
+
+		/* >>> Issue warning about oversized directory address */;
+
+		{ret = 0; goto ex;}
+	}
+	BURN_ALLOC_MEM(adr, char, BURN_DRIVE_ADR_LEN);
+        BURN_ALLOC_MEM(sys_adr, char, BURN_DRIVE_ADR_LEN);
+
+	dirpt = opendir(dir_adr);
+	if (dirpt == NULL)
+		{ret = 0; goto ex;}
+	strcpy(adr, dir_adr);
+        strcat(adr, "/");
+	namept = adr + strlen(dir_adr) + 1;
+	while(1) {
+		entry = readdir(dirpt);
+		if(entry == NULL)
+	break;
+		if (strlen(entry->d_name) + dirlen >= BURN_DRIVE_ADR_LEN)
+	continue;
+	  	strcpy(namept, entry->d_name);
+		if(lstat(adr, &link_stbuf) == -1)
+	continue;
+		if((link_stbuf.st_mode & S_IFMT) != S_IFLNK)
+	continue;
+		/* Determine rank and omit uninteresting ones */
+		for(name_rank= 0; name_rank < rank_count; name_rank++)
+			if(strncmp(namept, ranks[name_rank],
+					 strlen(ranks[name_rank])) == 0)
+		break;
+		/* we look for lowest rank */
+		if(name_rank >= rank_count ||
+		   name_rank > found_rank ||
+		   (name_rank == found_rank &&
+		    strcmp(namept, link_adr + dirlen) >= 0))
+	continue; 
+
+		/* Does name point to the same device as dev_adr ? */
+		ret= burn_drive_resolve_link(adr, sys_adr, &rec_count, 2);
+		if(ret < 0)
+			goto ex;
+		if(ret == 0)
+	continue;
+		if(strcmp(dev_adr, sys_adr) == 0) {
+			strcpy(link_adr, adr); 
+			found_rank= name_rank;
+		}
+	}
+	ret= 2;
+	if(found_rank < 0x7fffffff)
+		ret= 1;
+ex:;
+	if(dirpt != NULL)
+		closedir(dirpt);
+	BURN_FREE_MEM(adr);
+	BURN_FREE_MEM(sys_adr);
+	return(ret);
+}
+
+
 /** A pacifier function suitable for burn_abort.
     @param handle If not NULL, a pointer to a text suitable for printf("%s")
 */
@@ -3109,6 +3226,23 @@ int burn_disc_get_bd_spare_info(struct burn_drive *d,
 }
 
 
+/* ts B10801 : API */
+int burn_disc_get_phys_format_info(struct burn_drive *d, int *disk_category,
+                        char **book_name, int *part_version, int *num_layers,
+                        int *num_blocks, int flag)
+{
+	int ret;
+
+	if (burn_drive_get_drive_role(d) != 1)
+		return 0;
+	*disk_category = *part_version = *num_layers = *num_blocks = 0;
+	ret = mmc_get_phys_format_info(d, disk_category, book_name,
+				part_version, num_layers, num_blocks, 0);
+	return ret;
+}
+
+
+
 /* ts B10525 : API */
 int burn_disc_next_track_is_damaged(struct burn_drive *d, int flag)
 {
