@@ -244,7 +244,6 @@ int burn_write_close_track(struct burn_write_opts *o, struct burn_session *s,
 {
 	char msg[81];
 	struct burn_drive *d;
-	struct burn_track *t;
 
 	/* ts A61106 */
 #ifdef Libburn_experimental_no_close_tracK
@@ -252,7 +251,6 @@ int burn_write_close_track(struct burn_write_opts *o, struct burn_session *s,
 #endif
 
 	d = o->drive;
-	t = s->track[tnum];
 
 	d->busy = BURN_DRIVE_CLOSING_TRACK;
 
@@ -276,7 +274,7 @@ int burn_write_close_track(struct burn_write_opts *o, struct burn_session *s,
 
 
 /* ts A61030 */
-int burn_write_close_session(struct burn_write_opts *o, struct burn_session *s)
+int burn_write_close_session(struct burn_write_opts *o)
 {
 
 	/* ts A61106 */
@@ -666,19 +664,11 @@ int burn_write_leadout(struct burn_write_opts *o,
 int burn_write_session(struct burn_write_opts *o, struct burn_session *s)
 {
 	struct burn_drive *d = o->drive;
-	struct burn_track *prev = NULL, *next = NULL;
 	int i, ret;
 
 	d->rlba = 0;
 	burn_print(1, "    writing a session\n");
 	for (i = 0; i < s->tracks; i++) {
-		if (i > 0)
-			prev = s->track[i - 1];
-		if (i + 1 < s->tracks)
-			next = s->track[i + 1];
-		else
-			next = NULL;
-
 		if (!burn_write_track(o, s, i))
 			{ ret = 0; goto ex; }
 	}
@@ -687,7 +677,7 @@ int burn_write_session(struct burn_write_opts *o, struct burn_session *s)
 	ret = 1;
 ex:;
 	if (o->write_type == BURN_WRITE_TAO)
-		burn_write_close_session(o, s);
+		burn_write_close_session(o);
 	return ret;
 }
 
@@ -999,7 +989,7 @@ int burn_precheck_write(struct burn_write_opts *o, struct burn_disc *disc,
 {
 	enum burn_write_types wt;
 	struct burn_drive *d = o->drive;
-	char msg[160], *reason_pt;
+	char *msg = NULL, *reason_pt;
 	int no_media = 0;
 
 	reason_pt= reasons;
@@ -1056,13 +1046,18 @@ int burn_precheck_write(struct burn_write_opts *o, struct burn_disc *disc,
 			strcat(reasons, "write start address not supported, ");
 	} else {
 unsuitable_profile:;
-		sprintf(msg, "Unsuitable media detected. Profile %4.4Xh  %s",
-			d->current_profile, d->current_profile_text);
-		if (!silent)
+		msg = calloc(1, 160);
+		if (msg != NULL && !silent) {
+			sprintf(msg,
+			    "Unsuitable media detected. Profile %4.4Xh  %s",
+			    d->current_profile, d->current_profile_text);
 			libdax_msgs_submit(libdax_messenger, d->global_index,
 				0x0002011e,
 				LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
 				msg, 0, 0);
+		}
+		if (msg != NULL)
+			free(msg);
 		strcat(reasons, "no suitable media profile detected, ");
 		return 0;
 	}
@@ -1092,10 +1087,11 @@ int burn_disc_open_track_dvd_minus_r(struct burn_write_opts *o,
 					struct burn_session *s, int tnum)
 {
 	struct burn_drive *d = o->drive;
-	char msg[160];
+	char *msg = NULL;
 	int ret, lba, nwa;
 	off_t size;
 
+	BURN_ALLOC_MEM(msg, char, 160);
 	d->send_write_parameters(d, o);
 	ret = d->get_nwa(d, -1, &lba, &nwa);
 	sprintf(msg, 
@@ -1137,10 +1133,13 @@ int burn_disc_open_track_dvd_minus_r(struct burn_write_opts *o,
 				0x00020138,
 				LIBDAX_MSGS_SEV_FATAL, LIBDAX_MSGS_PRIO_HIGH,
 				msg, 0, 0);
-			return 0;
+			{ret = 0; goto ex;}
 		}
 	}
-	return 1;
+	ret = 1;
+ex:;
+	BURN_FREE_MEM(msg);
+	return ret;
 }
 
 
@@ -1149,10 +1148,11 @@ int burn_disc_open_track_dvd_plus_r(struct burn_write_opts *o,
 					struct burn_session *s, int tnum)
 {
 	struct burn_drive *d = o->drive;
-	char msg[160];
+	char *msg = NULL;
 	int ret, lba, nwa;
 	off_t size;
 
+	BURN_ALLOC_MEM(msg, char, 160);
 	ret = d->get_nwa(d, -1, &lba, &nwa);
 	sprintf(msg, 
 		"DVD+R pre-track %2.2d : get_nwa(%d), ret= %d , d->nwa= %d",
@@ -1183,16 +1183,18 @@ int burn_disc_open_track_dvd_plus_r(struct burn_write_opts *o,
 				0x00020138,
 				LIBDAX_MSGS_SEV_FATAL, LIBDAX_MSGS_PRIO_HIGH,
 				msg, 0, 0);
-			return 0;
+			{ret = 0; goto ex;}
 		}
 	}
-	return 1;
+	ret = 1;
+ex:;
+	BURN_FREE_MEM(msg);
+	return ret;
 }
 
 
 /* ts A70129 */
-int burn_disc_close_track_dvd_minus_r(struct burn_write_opts *o,
-					struct burn_session *s, int tnum)
+int burn_disc_close_track_dvd_minus_r(struct burn_write_opts *o, int tnum)
 {
 	struct burn_drive *d = o->drive;
 	char msg[80];
@@ -1249,7 +1251,7 @@ int burn_disc_finalize_dvd_plus_r(struct burn_write_opts *o)
 
 /* ts A70226 */
 int burn_disc_close_track_dvd_plus_r(struct burn_write_opts *o,
-			struct burn_session *s, int tnum, int is_last_track)
+			int tnum, int is_last_track)
 {
 	struct burn_drive *d = o->drive;
 	char msg[80];
@@ -1441,20 +1443,6 @@ int burn_dvd_write_track(struct burn_write_opts *o,
 
 	sectors = burn_track_get_sectors(t);
 	open_ended = burn_track_is_open_ended(t);
-	/* <<< */
-	{
-		char msg[160];
-
-		sprintf(msg,
-		 "DVD pre-track %2.2d : demand=%.f%s, cap=%.f\n",
-			tnum+1, (double) sectors * 2048.0,
-			(open_ended ? " (open ended)" : ""),
-			(double) d->media_capacity_remaining);
-		libdax_msgs_submit(libdax_messenger, d->global_index, 0x000002,
-				LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_ZERO,
-				msg, 0, 0);
-	}
-
 
 	/* (offset padding is done within sector_data()) */
 
@@ -1514,18 +1502,18 @@ int burn_dvd_write_track(struct burn_write_opts *o,
 	if (d->current_profile == 0x11 || d->current_profile == 0x14 ||
 	    d->current_profile == 0x15) {
 		/* DVD-R, DVD-RW Sequential, DVD-R/DL Sequential */
-		ret = burn_disc_close_track_dvd_minus_r(o, s, tnum);
+		ret = burn_disc_close_track_dvd_minus_r(o, tnum);
 		if (ret <= 0)
 			goto ex;
 	} else if (d->current_profile == 0x1b || d->current_profile == 0x2b) {
 		/* DVD+R , DVD+R/DL */
-		ret = burn_disc_close_track_dvd_plus_r(o, s, tnum,
+		ret = burn_disc_close_track_dvd_plus_r(o, tnum,
 							 is_last_track);
 		if (ret <= 0)
 			goto ex;
 	} else if (d->current_profile == 0x41) {
 		/* BD-R SRM */
-		ret = burn_disc_close_track_dvd_plus_r(o, s, tnum,
+		ret = burn_disc_close_track_dvd_plus_r(o, tnum,
 							 is_last_track);
 		if (ret <= 0)
 			goto ex;
@@ -1580,8 +1568,7 @@ int burn_disc_close_session_dvd_minus_rw(struct burn_write_opts *o,
 
 
 /* ts A70129 : for profile 0x11 DVD-R, 0x14 DVD-RW Seq, 0x15 DVD-R/DL Seq */
-int burn_disc_close_session_dvd_minus_r(struct burn_write_opts *o,
-					struct burn_session *s)
+int burn_disc_close_session_dvd_minus_r(struct burn_write_opts *o)
 {
 	struct burn_drive *d = o->drive;
 
@@ -1686,7 +1673,7 @@ int burn_dvd_write_session(struct burn_write_opts *o,
 		multi_mem = o->multi;
 		if (!is_last_session)
 			o->multi = 1;
-		ret = burn_disc_close_session_dvd_minus_r(o, s);
+		ret = burn_disc_close_session_dvd_minus_r(o);
 		o->multi = multi_mem;
 		if (ret <= 0)
 			return 0;
@@ -1746,7 +1733,7 @@ int burn_disc_setup_dvd_minus_rw(struct burn_write_opts *o,
 				struct burn_disc *disc)
 {
 	struct burn_drive *d = o->drive;
-	char msg[160];
+	char msg[60];
 	int ret;
 
 	d->nwa = 0;
@@ -1829,8 +1816,9 @@ int burn_dvd_write_sync(struct burn_write_opts *o,
 	off_t default_size = 0;
 	struct burn_drive *d = o->drive;
 	struct burn_track *t;
-	char msg[160];
+	char *msg = NULL;
 
+	BURN_ALLOC_MEM(msg, char, 160);
 	d->needs_close_session = 0;
 
 	/* buffer flush trigger for sector.c:get_sector() */
@@ -2021,9 +2009,10 @@ ex:;
 				0x0002017b, LIBDAX_MSGS_SEV_WARNING,
 				LIBDAX_MSGS_PRIO_ZERO, msg, 0, 0);
 	}
-
+	BURN_FREE_MEM(msg);
 	return ret;
 early_failure:;
+	BURN_FREE_MEM(msg);
 	return 0;
 }
 
@@ -2042,7 +2031,7 @@ int burn_stdio_open_write(struct burn_drive *d, off_t start_byte,
 
 	int fd = -1;
 	int mode = O_RDWR | O_CREAT | O_LARGEFILE;
-	char msg[160];
+	char msg[60];
 	off_t lseek_res;
 
 	if(d->drive_role == 4) {
@@ -2264,16 +2253,15 @@ int burn_stdio_slowdown(struct burn_drive *d, struct timeval *prev_time,
 int burn_stdio_write_track(struct burn_write_opts *o, struct burn_session *s,
 				int tnum, int flag)
 {
-	int open_ended, bufsize, ret, sectors, fd;
+	int open_ended, bufsize = 16 * 2048, ret, sectors;
 	struct burn_track *t = s->track[tnum];
 	struct burn_drive *d = o->drive;
-	char buf[16*2048];
+	char *buf = NULL;
 	int i, prev_sync_sector = 0;
 	struct buffer *out = d->buffer;
 	struct timeval prev_time;
 
-	bufsize = sizeof(buf);
-	fd = d->stdio_fd;
+	BURN_ALLOC_MEM(buf, char, bufsize);
 
 	sectors = burn_track_get_sectors(t);
 	burn_disc_init_track_status(o, s, tnum, sectors);
@@ -2324,6 +2312,7 @@ ex:;
 		burn_source_cancel(t->source);
 	if (t->end_on_premature_eoi == 2)
 		d->cancel = 1;
+	BURN_FREE_MEM(buf);
 	return ret;
 }
 
@@ -2531,6 +2520,8 @@ return crap.  so we send the command, then ignore the result.
 /*		print_cue(sheet);*/
 		if (o->write_type == BURN_WRITE_SAO)
 			d->send_cue_sheet(d, sheet);
+		if (sheet->data != NULL)
+			free(sheet->data);
 		free(sheet);
 
 		if (o->write_type == BURN_WRITE_RAW) {
@@ -2652,21 +2643,22 @@ int burn_random_access_write(struct burn_drive *d, off_t byte_address,
 {
 	int alignment = 0, start, upto, chunksize, err, fd = -1, ret;
 	char msg[81], *rpt;
-	struct buffer buf, *buffer_mem = d->buffer;
+	struct buffer *buf = NULL, *buffer_mem = d->buffer;
 
+	BURN_ALLOC_MEM(buf, struct buffer, 1);
 	if (d->released) {
 		libdax_msgs_submit(libdax_messenger,
 			d->global_index, 0x00020142,
 			LIBDAX_MSGS_SEV_FATAL, LIBDAX_MSGS_PRIO_HIGH,
 			"Drive is not grabbed on random access write", 0, 0);
-		return 0;
+		{ret = 0; goto ex;}
 	}
 	if(d->drive_role == 0) {
 		libdax_msgs_submit(libdax_messenger, d->global_index,
 			0x00020146,
 			LIBDAX_MSGS_SEV_FATAL, LIBDAX_MSGS_PRIO_HIGH,
 			"Drive is a virtual placeholder (null-drive)", 0, 0);
-		return 0;
+		{ret = 0; goto ex;}
 	}
 	if(d->drive_role == 4) {
 		libdax_msgs_submit(libdax_messenger, d->global_index,
@@ -2674,7 +2666,7 @@ int burn_random_access_write(struct burn_drive *d, off_t byte_address,
 			LIBDAX_MSGS_SEV_FAILURE, LIBDAX_MSGS_PRIO_HIGH,
 			"Pseudo-drive is a read-only file. Cannot write.",
 			0, 0);
-		return 0;
+		{ret = 0; goto ex;}
 	}
 
 	if(d->drive_role == 2 || d->drive_role == 5)
@@ -2693,7 +2685,7 @@ int burn_random_access_write(struct burn_drive *d, off_t byte_address,
 			0x00020125,
 			LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
 			"Write start address not supported", 0, 0);
-		return 0;
+		{ret = 0; goto ex;}
 	}
 	if ((byte_address % alignment) != 0) {
 		sprintf(msg,
@@ -2703,7 +2695,7 @@ int burn_random_access_write(struct burn_drive *d, off_t byte_address,
 			0x00020126,
 			LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
 			msg, 0, 0);
-		return 0;
+		{ret = 0; goto ex;}
 	}
 	if ((data_count % alignment) != 0) {
 		sprintf(msg,
@@ -2713,22 +2705,22 @@ int burn_random_access_write(struct burn_drive *d, off_t byte_address,
 			0x00020141,
 			LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
 			msg, 0, 0);
-		return 0;
+		{ret = 0; goto ex;}
 	}
 	if (d->busy != BURN_DRIVE_IDLE) {
 		libdax_msgs_submit(libdax_messenger,
 			d->global_index, 0x00020140,
 			LIBDAX_MSGS_SEV_FATAL, LIBDAX_MSGS_PRIO_HIGH,
 			"Drive is busy on attempt to write random access",0,0);
-		return 0;
+		{ret = 0; goto ex;}
 	}
 	if(d->drive_role != 1) {
 		fd = burn_stdio_open_write(d, byte_address, 2048, 0);
 		if (fd == -1)
-			return 0;
+			{ret = 0; goto ex;}
 	}
 	d->busy = BURN_DRIVE_WRITING_SYNC;
-	d->buffer = &buf;
+	d->buffer = buf;
 
 	start = byte_address / 2048;
 	upto = start + data_count / 2048;
@@ -2755,7 +2747,7 @@ int burn_random_access_write(struct burn_drive *d, off_t byte_address,
 			d->busy = BURN_DRIVE_IDLE;
 			if(fd >= 0)
 				close(fd);
-			return (-(start * 2048 - byte_address));
+			{ret = -(start * 2048 - byte_address); goto ex;}
 		}
 	}
 
@@ -2773,6 +2765,95 @@ int burn_random_access_write(struct burn_drive *d, off_t byte_address,
 		close(fd);
 	d->buffer = buffer_mem;
 	d->busy = BURN_DRIVE_IDLE;
-	return 1;
+	ret = 1;
+ex:
+	BURN_FREE_MEM(buf);
+	return ret;
 }
 
+
+/* ts B10527 */
+/* @param bit0= force close, even if no damage was seen
+*/
+int burn_disc_close_damaged(struct burn_write_opts *o, int flag)
+{
+	struct burn_drive *d;
+	int ret;
+	enum burn_drive_status busy;
+
+	d = o->drive;
+	busy = d->busy;
+
+	if (busy != BURN_DRIVE_IDLE) {
+		libdax_msgs_submit(libdax_messenger,
+			d->global_index, 0x00020106,
+			LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
+			"Drive is busy on attempt to close damaged session",
+			0, 0);
+		{ret = 0; goto ex;}
+	}
+	if (!((d->next_track_damaged & 1) || (flag & 1))) {
+		libdax_msgs_submit(libdax_messenger,
+			d->global_index, 0x00020187,
+			LIBDAX_MSGS_SEV_NOTE, LIBDAX_MSGS_PRIO_HIGH,
+			"Track not marked as damaged. No action taken.",
+			0, 0);
+		{ret = 0; goto ex;}
+	}
+	d->busy = BURN_DRIVE_WRITING;
+
+	if (d->current_profile == 0x09 || d->current_profile == 0x0a) {
+		/* Close CD track and session */
+		o->write_type = BURN_WRITE_TAO; /* no action without TAO */
+
+		/* Send mode page 5 */;
+		d->send_write_parameters(d, o);
+
+		ret = burn_write_close_session(o);
+		if (ret <= 0)
+			goto ex;
+
+	} else if(d->current_profile == 0x11 || d->current_profile == 0x14) {
+		/* Close DVD-R[W] track and session */
+		o->write_type = BURN_WRITE_TAO; /* no action without TAO */
+
+		/* Send mode page 5 */;
+		d->send_write_parameters(d, o);
+
+		ret = burn_disc_close_track_dvd_minus_r(o, 0);
+		if (ret <= 0)
+			goto ex;
+		ret = burn_disc_close_session_dvd_minus_r(o);
+		if (ret <= 0)
+			goto ex;
+
+	} else if(d->current_profile == 0x1b || d->current_profile == 0x2b) {
+		/* Close DVD+R track and session */
+		ret = burn_disc_close_track_dvd_plus_r(o, d->last_track_no, 1);
+		if (ret <= 0)
+			goto ex;
+
+	} else if(d->current_profile == 0x41) {
+		/* Close BD-R track and session */
+		ret = burn_disc_close_track_dvd_plus_r(o, d->last_track_no, 1);
+		if (ret <= 0)
+			goto ex;
+
+	} else {
+		libdax_msgs_submit(libdax_messenger,
+			d->global_index, 0x00020188,
+			LIBDAX_MSGS_SEV_FAILURE, LIBDAX_MSGS_PRIO_HIGH,
+			"Cannot close damaged track on given media type",
+			0, 0);
+		{ret = 0; goto ex;}
+
+	}
+	ret = 1;
+ex:;
+	d->busy = busy;
+	/* Record with drive that repair was attempted */
+	d->next_track_damaged &= ~1;
+	return ret;
+}
+
+
