@@ -564,7 +564,9 @@ int isoburn_start_emulation(struct isoburn *o, int flag)
    isoburn_set_start_byte(o, size, 0);
    if(!(flag & 1))
      o->fabricated_disc_status= BURN_DISC_APPENDABLE;
- } else if (!strncmp((char*)pvm->std_identifier, "CDXX1", 5)) {
+ } else if (strncmp((char*)pvm->std_identifier, "CDXX1", 5) == 0 ||
+            (strncmp((char*)pvm->std_identifier, "CDxx1", 5) == 0 &&
+             pvm->vol_desc_type[0] == 'x')) {
 
    /* empty image */
    isoburn_set_start_byte(o, o->zero_nwa * 2048, 0);
