@@ -247,9 +247,8 @@ void iso_msg_debug(int imgid, const char *fmt, ...)
 {
     char *msg = NULL;
     va_list ap;
-    int ret;
 
-    LIBISO_ALLOC_MEM(msg, char, MAX_MSG_LEN);
+    LIBISO_ALLOC_MEM_VOID(msg, char, MAX_MSG_LEN);
     va_start(ap, fmt);
     vsnprintf(msg, MAX_MSG_LEN, fmt, ap);
     va_end(ap);
