@@ -800,7 +800,7 @@ int Xorriso_toc(struct XorrisO *xorriso, int flag)
  int track_count= 0, session_no, track_no, profile_no= -1, track_size;
  int last_track_start= 0, last_track_size= -1, num_data= 0, is_data= 0;
  int is_inout_drive= 0, drive_role, status, num_formats, emul_lba;
- int num_payload= 0, not_reconizable= 0;
+ int not_recognizable= 0, start_lba, end_lba;
  char profile_name[80],*respt,*devadr, *typetext= "";
  struct burn_toc_entry toc_entry;
  struct burn_drive_info *dinfo;
@@ -816,6 +816,11 @@ int Xorriso_toc(struct XorrisO *xorriso, int flag)
  int image_blocks= 0;
  char volume_id[33];
  struct burn_toc_entry next_toc_entry;
+ int disk_category, part_version, num_layers, num_blocks;
+ char *book_name;
+/*
+ int num_payload= 0, num_wasted= 0, num_nondata= 0;
+*/
 
  ret= Xorriso_get_drive_handles(xorriso, &dinfo, &drive,
                                 "on attempt to print Table Of Content",
@@ -839,7 +844,7 @@ int Xorriso_toc(struct XorrisO *xorriso, int flag)
    {ret= 1; goto ex;}
  sprintf(respt, "Drive type   : vendor '%s' product '%s' revision '%s'\n",
         dinfo[0].vendor, dinfo[0].product, dinfo[0].revision);
- if((flag & 32) | !(flag & 1))
+ if((flag & 32) || !(flag & 1))
    Xorriso_toc_line(xorriso, flag & 8);
  if(flag & 32)
    {ret= 1; goto ex;}
@@ -866,7 +871,7 @@ int Xorriso_toc(struct XorrisO *xorriso, int flag)
    strcat(respt, "\n");
  } else {
     sprintf(respt+strlen(respt), "is not recognizable\n");
-    not_reconizable= 1;
+    not_recognizable= 1;
  }
  Xorriso_toc_line(xorriso, flag & 8);
 
@@ -878,7 +883,7 @@ int Xorriso_toc(struct XorrisO *xorriso, int flag)
 
  sprintf(respt, "Media status : ");
  if (s == BURN_DISC_FULL) {
-   if(not_reconizable)
+   if(not_recognizable)
      sprintf(respt+strlen(respt), "is not recognizable\n");
    else
      sprintf(respt+strlen(respt), "is written , is closed");
@@ -909,6 +914,59 @@ int Xorriso_toc(struct XorrisO *xorriso, int flag)
  strcat(respt, "\n");
  Xorriso_toc_line(xorriso, flag & 8);
 
+ if((s == BURN_DISC_FULL || s == BURN_DISC_APPENDABLE ||
+     s == BURN_DISC_BLANK) && !(flag & 1)) {
+   ret= burn_get_read_capacity(drive, &num_data, 0);
+   if(ret != 1 || s == BURN_DISC_BLANK)
+     num_data= 0;
+   num_free= isoburn_disc_available_space(drive, NULL) / 2048; 
+   nwa= -1;
+   if (s == BURN_DISC_APPENDABLE) {
+     ret= isoburn_disc_track_lba_nwa(drive, NULL, 0, &lba, &nwa);
+     if(ret <= 0)
+       nwa= -1;
+   } else if(s == BURN_DISC_BLANK) {
+     ret= isoburn_disc_track_lba_nwa(drive, NULL, 0, &lba, &nwa);
+     if(ret == 1) {
+       num_free+= nwa;
+       nwa= 0;
+     }
+   }
+   lba= num_data + num_free;
+   if(nwa >= 0) {
+     lba= nwa + num_free;
+     if(nwa < num_data)
+       num_data= nwa;
+   }
+
+   /* If closed CD-RW : obtain ATIP lead out address */
+   if(profile_no == 0x0a) {
+     ret= burn_disc_read_atip(drive);
+     if(ret < 0)
+       goto ex;
+     ret= burn_drive_get_start_end_lba(drive, &start_lba, &end_lba, 0);
+     if(s == BURN_DISC_FULL && ret == 1) {
+       lba= end_lba - 2;
+     } else {
+       if(ret == 1 && end_lba  - 2 > lba) {
+         sprintf(xorriso->info_text,
+                 "ATIP end_lba %d > overall %d", end_lba, lba);
+         Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "WARNING", 0);
+       }
+     }
+   } else if(profile_no == 0x14) {
+     ret= burn_disc_get_phys_format_info(drive, &disk_category,
+                                         &book_name, &part_version,
+                                         &num_layers, &num_blocks, 0);
+     if(ret == 1 && num_blocks > lba)
+       lba= num_blocks;
+   }
+
+   sprintf(respt, "Media blocks : %d readable , %d writable , %d overall\n",
+                  num_data, (int) num_free, lba);
+   Xorriso_toc_line(xorriso, flag & 8);
+ }
+
  if(s == BURN_DISC_BLANK) {
    sprintf(respt, "Media summary: 0 sessions, 0 data blocks, 0 data");
    num_free= isoburn_disc_available_space(drive, NULL); 
@@ -938,15 +996,15 @@ int Xorriso_toc(struct XorrisO *xorriso, int flag)
    Xorriso_process_msg_queues(xorriso,0);
    if(drive_role == 5 && s == BURN_DISC_APPENDABLE) {
      ret= burn_disc_track_lba_nwa(drive, NULL, 0, &lba, &nwa);
-     if(ret == 1)
-       num_data= nwa;
+     if(ret != 1)
+       lba= 0;
    } else {
      ret= isoburn_get_min_start_byte(drive, &start_byte, 0);
      nwa= start_byte / 2048;
      if(ret<=0) {
        Xorriso_process_msg_queues(xorriso,0);
        if(flag&1)
-         {ret= 0; goto ex;}
+         {ret= 1; goto ex;}
        sprintf(xorriso->info_text, "Cannot obtain Table Of Content");
        Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
        {ret= 0; goto ex;}
@@ -965,20 +1023,29 @@ int Xorriso_toc(struct XorrisO *xorriso, int flag)
              1, 0, image_blocks, volume_id);
      nwa= image_blocks;
    } else {
-     nwa= num_data;
      ret= burn_disc_get_formats(drive, &status, &size, &dummy,
                                 &num_formats);
-     if(ret>0 && status==BURN_FORMAT_IS_FORMATTED)
-       nwa= size/2048;
+     if(ret <= 0 || status != BURN_FORMAT_IS_FORMATTED)
+       size= 0;
+     if(size <= 0) {
+       ret= burn_get_read_capacity(drive, &num_data, 0);
+       if(ret == 1)
+         size= ((off_t) num_data) * (off_t) 2048; 
+     }
+     nwa= lba + size / 2048;
+     num_data= nwa - lba;
      sprintf(respt, "%13s: %3d , %9d , %9ds , \n",
-             typetext, 1, num_data, nwa);
+             typetext, 1, lba, num_data);
    } 
    if(!(flag&1))
      Xorriso_toc_line(xorriso, flag & 8);
    last_track_start= lba;
+/*
    num_payload= num_data= last_track_size= nwa;
+*/
    num_sessions= 1;
  } else {
+   num_data= 0;
    sessions= isoburn_toc_disc_get_sessions(disc, &num_sessions);
    for (session_no= 0; session_no<num_sessions && !(xorriso->request_to_abort);
         session_no++) {
@@ -1043,8 +1110,10 @@ int Xorriso_toc(struct XorrisO *xorriso, int flag)
        } 
        if(!(flag&1))
          Xorriso_toc_line(xorriso, flag & 8);
+/*
        if(track_no>0)
          num_payload+= lba - last_track_start;
+*/
        last_track_start= lba;
        if((toc_entry.control&7)>=4) /* data track */
          is_data= 1;
@@ -1058,7 +1127,9 @@ int Xorriso_toc(struct XorrisO *xorriso, int flag)
        lba= burn_msf_to_lba(toc_entry.pmin, toc_entry.psec, toc_entry.pframe);
      }
      last_track_size= lba - last_track_start;
+/*
      num_payload+= last_track_size;
+*/
      if(is_data)
        num_data+= last_track_size;
    }
@@ -1066,8 +1137,6 @@ int Xorriso_toc(struct XorrisO *xorriso, int flag)
  if(xorriso->request_to_abort)
    {ret= 1; goto ex;}
 /*
- int num_wasted= 0, num_nondata= 0;
-
  num_wasted= lba - num_payload;
  num_nondata= lba - num_data;
 */
@@ -1090,6 +1159,12 @@ int Xorriso_toc(struct XorrisO *xorriso, int flag)
        Xorriso_toc_line(xorriso, flag & 8);
    }
  }
+ if(profile_no == 0x41 && num_sessions >= 300) { 
+   sprintf(xorriso->info_text,
+           "Sequential BD-R media now contains %d sessions. It is likely to soon fail writing.",
+           num_sessions);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "WARNING", 0);
+ }
 
  if (disc!=NULL)
    isoburn_toc_disc_free(disc);
@@ -1100,9 +1175,11 @@ ex:;
 }
 
 
+/* @param flag bit0= try to find 'meaningful' links for enumerated devices
+*/
 int Xorriso_show_devices(struct XorrisO *xorriso, int flag)
 {
- char *adr= NULL;
+ char *adr= NULL, *link_adr= NULL, *adrpt;
  int i, j, max_dev_len= 1, pad, ret;
  struct burn_drive_info *drive_list= NULL;
  unsigned int drive_count;
@@ -1110,6 +1187,7 @@ int Xorriso_show_devices(struct XorrisO *xorriso, int flag)
  struct stat stbuf;
 
  Xorriso_alloc_meM(adr, char, BURN_DRIVE_ADR_LEN);
+ Xorriso_alloc_meM(link_adr, char, BURN_DRIVE_ADR_LEN);
 
  sprintf(xorriso->info_text, "Beginning to scan for devices ...\n");
  Xorriso_info(xorriso,0);
@@ -1139,8 +1217,16 @@ int Xorriso_show_devices(struct XorrisO *xorriso, int flag)
    if(burn_drive_get_adr(&(drive_list[i]), adr)<=0)
      strcpy(adr, "-get_adr_failed-");
    Xorriso_process_msg_queues(xorriso,0);
-   if((int) strlen(adr) > max_dev_len)
-     max_dev_len= strlen(adr);
+   adrpt= adr;
+   if(flag & 1) {
+     ret= burn_lookup_device_link(adr, link_adr, "/dev", NULL, 0, 0);
+     if(ret < 0)
+       goto ex;
+     if(ret == 1)
+       adrpt= link_adr;
+   }
+   if((int) strlen(adrpt) > max_dev_len)
+     max_dev_len= strlen(adrpt);
  }
  for(i= 0; i < (int) drive_count && !(xorriso->request_to_abort); i++) {
    if(burn_drive_get_adr(&(drive_list[i]), adr)<=0)
@@ -1157,8 +1243,16 @@ int Xorriso_show_devices(struct XorrisO *xorriso, int flag)
      if(stbuf.st_mode&S_IROTH) perms[4]= 'r';
      if(stbuf.st_mode&S_IWOTH) perms[5]= 'w';
    }
-   sprintf(respt, "%d  -dev '%s' ", i, adr);
-   pad= max_dev_len-strlen(adr);
+   adrpt= adr;
+   if(flag & 1) {
+     ret= burn_lookup_device_link(adr, link_adr, "/dev", NULL, 0, 0);
+     if(ret < 0)
+       goto ex;
+     if(ret == 1)
+       adrpt= link_adr;
+   }
+   sprintf(respt, "%d  -dev '%s' ", i, adrpt);
+   pad= max_dev_len-strlen(adrpt);
    if(pad>0)
      for(j= 0; j<pad; j++)
        strcat(respt, " ");
@@ -1170,10 +1264,11 @@ int Xorriso_show_devices(struct XorrisO *xorriso, int flag)
  Xorriso_info(xorriso,0);
 
  burn_drive_info_free(drive_list);
- Xorriso_process_msg_queues(xorriso,0);
  ret= 1;
 ex:;
+ Xorriso_process_msg_queues(xorriso,0);
  Xorriso_free_meM(adr);
+ Xorriso_free_meM(link_adr);
  return(ret);
 }
 
@@ -1284,11 +1379,55 @@ ex:;
 }
 
 
+int Xorriso_choose_speed_factor(struct XorrisO *xorriso,
+                                int speed, int profile,
+                                double *speed_factor, char **speed_unit,
+                                int flag)
+{
+ double cd_factor = 75.0 * 2352;
+ double cd_speed_tolerance= 1.5, cd_speed_add;
+ int int_cd_speed, i;
+ static int cd_speed_list[]= {8, 10, 12, 16, 24, 32, 40, 48, 52, 0};
+
+ *speed_unit= "D";  
+ *speed_factor= 1385000.0;
+
+ /* Does this look like an integer CD speed ? */
+ int_cd_speed= ((double) speed) * 1000.0 / cd_factor;
+ cd_speed_add= cd_speed_tolerance * (double) int_cd_speed;
+ int_cd_speed= (((double) speed) + cd_speed_add)
+                 * 1000.0 / cd_factor;
+ if(abs((int) ((double) int_cd_speed) * cd_factor / 1000.0 -
+              ((double) speed)) > 2 * cd_speed_add ||
+    int_cd_speed > 64)
+   int_cd_speed= 0;
+ if(int_cd_speed > 7) {
+   for(i= 0; cd_speed_list[i]; i++)
+     if(int_cd_speed == cd_speed_list[i])
+   break;
+   if(cd_speed_list[i] == 0)
+     int_cd_speed= 0;
+ }
+      
+ if(((profile < 0x08 || profile >= 0x100 || profile == 0x10 || profile == 0x40)
+    && int_cd_speed) ||
+    (profile >= 0x08 && profile <= 0x0a)) {
+   *speed_unit= "C";
+   *speed_factor= cd_factor;
+ } else if(profile >= 0x40 && profile <= 0x43) {
+   *speed_unit= "B";
+   *speed_factor= 4495625.0;
+ }
+ return(1);
+}
+
+
 /* @return <=0 error, 1 success
 */
 int Xorriso_list_speeds(struct XorrisO *xorriso, int flag)
 {
- int ret, high= -1, low= 0x7fffffff, int_cd_speed, is_cd= 0;
+ int ret, high= -1, low= 0x7fffffff, is_cd= 0, i;
+ int recent_profile= 0;
  char *respt, *speed_unit= "D";
  double speed_factor= 1385000.0, cd_factor= 75.0 * 2352;
  struct burn_drive_info *dinfo;
@@ -1304,18 +1443,29 @@ int Xorriso_list_speeds(struct XorrisO *xorriso, int flag)
  if(ret == 2)
    goto ex;
  ret= burn_drive_get_speedlist(drive, &speed_list);
- if(ret <= 0) {
+ if(ret < 0) {
    sprintf(xorriso->info_text, "Cannot obtain speed list info");
-   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
    ret= 0; goto ex;
  }
  ret= Xorriso_toc(xorriso, 3);
- if(ret<=0)
-   goto ex;
+ if(ret<=0) {
+   sprintf(xorriso->info_text,
+           "Cannot obtain overview of drive and media content");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
+   ret= 0; goto ex;
+ }
 
  for (item= speed_list; item != NULL; item= item->next) {
+
+   sprintf(xorriso->info_text, "speed= %5dk , source= %d",
+           item->write_speed, item->source);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
+
    if(item->profile_loaded >= 0x08 && item->profile_loaded <= 0x0a)
-     is_cd= 1;
+     is_cd= item->profile_loaded;
+   if(item->profile_loaded > 0)
+     recent_profile= item->profile_loaded;
    if(item->source == 1) {
      /* CD mode page 2Ah : report only if not same speed by GET PERFORMANCE */
      for(other= speed_list; other != NULL; other= other->next)
@@ -1324,25 +1474,9 @@ int Xorriso_list_speeds(struct XorrisO *xorriso, int flag)
      if(other != NULL)
  continue;
    }
-   speed_unit= "D";  
-   speed_factor= 1385000.0;
-
-   /* Does this look like an integer CD speed ? */
-   int_cd_speed= ((double) item->write_speed) * 1000.0 / cd_factor;
-   if(abs((int) ((double) int_cd_speed) * cd_factor / 1000.0 -
-                ((double) item->write_speed)) > 5 ||
-      int_cd_speed > 64)
-     int_cd_speed= 0;
-      
-   if(((item->profile_loaded < 0x08 || item->profile_loaded >= 0x100) &&
-       int_cd_speed) ||
-      (item->profile_loaded >= 0x08 && item->profile_loaded <= 0x0a)) {
-     speed_unit= "C";
-     speed_factor= 75.0 * 2352.0;
-   } else if(item->profile_loaded >= 0x41 && item->profile_loaded <= 0x43) {
-     speed_unit= "B";
-     speed_factor= 4495625.0;
-   }
+   Xorriso_choose_speed_factor(xorriso, item->write_speed,
+                               item->profile_loaded,
+                               &speed_factor, &speed_unit, 0);
    sprintf(respt, "Write speed  : ");
    sprintf(respt + strlen(respt), " %5dk , %4.1fx%s\n",
            item->write_speed,
@@ -1361,41 +1495,48 @@ int Xorriso_list_speeds(struct XorrisO *xorriso, int flag)
    if(ret < 0)
      goto ex;
    if(ret > 0) {
-     ret= burn_drive_get_min_write_speed(drive);
-     if(ret > 0) {
-       if(ret > low) {
-         sprintf(respt, "Write speed l: ");
-         sprintf(respt + strlen(respt), " %5dk , %4.1fx%s\n",
-                 ret, ((double) ret) * 1000.0 / cd_factor, "C");
-         Xorriso_result(xorriso,0);
-       }
-       if(ret > high)
-         high= ret;
-       low= ret;
-     } 
-     ret= burn_drive_get_write_speed(drive);
-     if(ret > 0) {
-       if(ret < high) {
-         sprintf(respt, "Write speed h: ");
-         sprintf(respt + strlen(respt), " %5dk , %4.1fx%s\n",
-                 ret, ((double) ret) * 1000.0 / cd_factor, "C");
-         Xorriso_result(xorriso,0);
-       }
-       high= ret;
-       if(ret < low)
-         low= ret;
-     } 
+     for(i= 0; i < 2; i++) {
+       if(i == 0)
+         ret= burn_drive_get_min_write_speed(drive);
+       else
+         ret= burn_drive_get_write_speed(drive);
+       if(ret > 0) {
+         if(ret < low || (i == 0 && ret != low)) {
+           sprintf(respt, "Write speed l: ");
+           sprintf(respt + strlen(respt), " %5dk , %4.1fx%s\n",
+                   ret, ((double) ret) * 1000.0 / cd_factor, "C");
+           Xorriso_result(xorriso,0);
+           low= ret;
+         }
+         if(ret > high || (i == 1 && ret != high)) {
+           sprintf(respt, "Write speed h: ");
+           sprintf(respt + strlen(respt), " %5dk , %4.1fx%s\n",
+                   ret, ((double) ret) * 1000.0 / cd_factor, "C");
+           Xorriso_result(xorriso,0);
+           high= ret;
+         }
+       } 
+     }
    }
  }
  if(high > -1) {
+   Xorriso_choose_speed_factor(xorriso, low, recent_profile,
+                               &speed_factor, &speed_unit, 0);
    sprintf(respt, "Write speed L: ");
    sprintf(respt + strlen(respt), " %5dk , %4.1fx%s\n",
            low, ((double) low) * 1000.0 / speed_factor, speed_unit);
    Xorriso_result(xorriso,0);
+   Xorriso_choose_speed_factor(xorriso, low, recent_profile,
+                               &speed_factor, &speed_unit, 0);
    sprintf(respt, "Write speed H: ");
    sprintf(respt + strlen(respt), " %5dk , %4.1fx%s\n",
            high, ((double) high) * 1000.0 / speed_factor, speed_unit);
    Xorriso_result(xorriso,0);
+ } else {
+   sprintf(xorriso->info_text,
+           "Could not get any write speed information from drive");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
+   ret= 0; goto ex;
  }
  ret= 1;
 ex:;
@@ -2290,7 +2431,7 @@ int Xorriso_check_media(struct XorrisO *xorriso, struct SpotlisT **spotlist,
  } else if(mode == 0) { /* track by track */
    isoburn_disc= isoburn_toc_drive_get_disc(drive);
    if(isoburn_disc == NULL)
-     goto no_content_visible;
+     goto libburn_whole_disc;
    isoburn_sessions=
                     isoburn_toc_disc_get_sessions(isoburn_disc, &num_sessions);
    for(i= 0; i < num_sessions; i++) {
@@ -2361,27 +2502,33 @@ int Xorriso_check_media(struct XorrisO *xorriso, struct SpotlisT **spotlist,
 
  } else if(mode == 1) { /* isoburn disc capacity */
    isoburn_disc= isoburn_toc_drive_get_disc(drive);
-   if(isoburn_disc == NULL) {
-no_content_visible:;
-     Xorriso_process_msg_queues(xorriso,0);
-     sprintf(xorriso->info_text, "No content detected on media");
-     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
-     {ret= 0; goto ex;}
-   }
+   if(isoburn_disc == NULL)
+     goto libburn_whole_disc;
    blocks= media_blocks= isoburn_toc_disc_get_sectors(isoburn_disc);
    if(start_lba >= 0)
      blocks-= start_lba;
    if(media_blocks <= 0)
-     goto no_content_visible;
+     goto libburn_whole_disc;
    xorriso->pacifier_total= blocks;
    ret= Xorriso_check_interval(xorriso, *spotlist, job, start_lba, blocks,
                                read_chunk, 0, 0);
    if(ret <= 0)
      goto ex;
  } else if(mode == 2) {
-
-   /* >>> single sweep over libburn media capacity */;
-
+libburn_whole_disc:;
+   /* single sweep over libburn media capacity */
+   ret= burn_get_read_capacity(drive, &blocks, 0);
+   if(ret <= 0) {
+     Xorriso_process_msg_queues(xorriso,0);
+     sprintf(xorriso->info_text, "No content detected on media");
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     {ret= 0; goto ex;}
+   }
+   xorriso->pacifier_total= blocks;
+   ret= Xorriso_check_interval(xorriso, *spotlist, job, 0, blocks,
+                               read_chunk, 0, 0);
+   if(ret <= 0)
+     goto ex;
  }
 
  Xorriso_pacifier_callback(xorriso, "sectors examined",
