@@ -38,6 +38,10 @@ struct Mx1 {
  
 };
 
+int Mx1_substitute(struct Mx1 *m, char line_in[256], char line_out[256],
+                   int raw, int upto, int flag);
+
+
 
 int Mx1_init(struct Mx1 *m, char *prog, int flag)
 {
@@ -78,64 +82,213 @@ int Mx1__get_word(char *line, char word[256], char **remainder, int flag)
 }
 
 
-int Mx1_substitute(struct Mx1 *m, char line_in[256], char line_out[256],
-                   int raw, int flag)
+int Mx1_is_wrap(struct Mx1 *m, char wraps[][20], char *start, char **found,
+                int flag)
 {
- char *rpt, *wpt, *ept;
- int l;
+ int i;
+
+ for(i= 0; wraps[i][0] != 0; i++)
+   if(strncmp(start, wraps[i], strlen(wraps[i])) == 0)
+ break;
+ if(wraps[i][0] != 0) {
+   if(found != NULL)
+      *found= wraps[i];
+   return(1);
+ }
+ return(0);
+}
+
+
+int Mx1_is_bold_wrap(struct Mx1 *m, char *start, char **found, int flag)
+{
+ int ret;
+ static char bold_wraps[][20]= {
+   "@b{", "@dfn{", "@emph{", "@strong{", "@command{",
+   "" };
+ 
+ ret= Mx1_is_wrap(m, bold_wraps, start, found, 0);
+ return(ret);
+}
+
+
+int Mx1_is_normal_wrap(struct Mx1 *m, char *start, char **found, int flag)
+{
+ int ret;
+ static char normal_wraps[][20]= {
+   "@var{", "@code{", "@i{", "@abbr{", "@file{", "@option{", "@samp{", "@r{",
+    "" };
+ 
+ ret= Mx1_is_wrap(m, normal_wraps, start, found, 0);
+ return(ret);
+}
+
+
+int Mx1_is_ignored_wrap(struct Mx1 *m, char *start, char **found, int flag)
+{
+ int ret;
+ static char ignored_wraps[][20]= {
+   "@ref{", "@xref{",
+    "" };
+ 
+ ret= Mx1_is_wrap(m, ignored_wraps, start, found, 0);
+ return(ret);
+}
+
+
+int Mx1_is_any_wrap(struct Mx1 *m, char *start, char **found, int flag)
+{
+ int ret;
+
+ ret= Mx1_is_bold_wrap(m, start, found, 0);
+ if(ret > 0)
+   return(1);
+ ret= Mx1_is_normal_wrap(m, start, found, 0);
+ if(ret > 0)
+   return(2);
+ ret= Mx1_is_ignored_wrap(m, start, found, 0);
+ if(ret > 0)
+   return(3);
+ return(0);
+}
+
+
+/* @param flag bit0= recursion
+               bit1= drop content of brackets
+*/
+int Mx1_rewrap(struct Mx1 *m, char **read_pt, char **write_pt,
+                char *write_base, char *envelope,
+                char *front, char *back, int flag)
+{
+ char *rpt, *wpt, *ept, content[256], msg[256];
+ int l, ret;
+
+ rpt= *read_pt;
+ wpt= *write_pt;
+
+ ept= strchr(rpt, '}');
+ if(ept == NULL) {
+   sprintf(msg, "No closing bracket found for '%s'", envelope);
+   Mx1_report_error(m, msg, 0);
+   return(-1);
+ }
+ /* Mapped {...} content is subject to the rules except {...} mapping. */
+ l= ept - rpt;
+ if(flag & 2)
+   l= 0;
+ if(l > 0) {
+   ret= Mx1_substitute(m, rpt, content, 0, l, 1);
+   if(ret <= 0)
+     return(ret);
+   l= strlen(content);
+ }
+ if((wpt - write_base) + l + strlen(front) + strlen(back) > 255) {
+   Mx1_report_error(m, "Line length overflow while text substitution", 0);
+   return(-1);
+ }
+ strcpy(wpt, front);
+ wpt+= strlen(front);
+ if(l > 0)
+   strncpy(wpt, content, l);
+ wpt+= l;
+ strcpy(wpt, back);
+ wpt+= strlen(back);
+
+ (*read_pt)+= ept - rpt;
+ (*write_pt)= wpt;
+ return(1);
+}
+
+
+/* @param flag bit0= recursion
+*/
+int Mx1_substitute(struct Mx1 *m, char line_in[256], char line_out[256],
+                   int raw, int upto, int flag)
+{
+ char *rpt, *wpt, *found;
+ int ret, typ= 0;
 
  wpt= line_out;
- for(rpt= line_in; *rpt != 0; rpt++) {
+ for(rpt= line_in; rpt - line_in < upto && *rpt != 0; rpt++) {
    if(rpt - line_in < raw) {
      *(wpt++)= *rpt;
  continue;
    } 
    if(*rpt == '@') {
-     if(strncmp(rpt, "@strong{", 8) == 0) {
-       /* @strong{-...} gets mapped to \fB\-...\fR . */;
-       /* @strong{... } gets mapped to \fB...\fR . */
-       ept= strchr(rpt, '}');
-       if(ept == NULL) {
-         Mx1_report_error(m, "No closing bracket found for '@strong{'", 0);
-         return(-1);
-       }
-       l= ept - rpt - 8;
-       if((wpt - line_out) + l + 6 + (rpt[8] == '-') > 255)
-         goto overflow;
-       strcpy(wpt, "\\fB");
-       wpt+= 3;
-       if(rpt[8] == '-')
-         *(wpt++)= '\\';
-       strncpy(wpt, rpt + 8, l);
-       wpt+= l;
-       strcpy(wpt, "\\fR");
-       wpt+= 3;
-       rpt+= ept - rpt;
+     typ= 0;
+     if(!(flag & 1))
+       typ= Mx1_is_any_wrap(m, rpt, &found, 0);
+     if(typ == 1) {
+       /* @b{...}, @command{...}, @dfn{...}, @emph{...}, @strong{...}
+            get mapped to \fB...\fR .
+       */
+       rpt+= strlen(found);
+       ret= Mx1_rewrap(m, &rpt, &wpt, line_out,
+                        found , "\\fB", "\\fR", flag & 1);
+       if(ret <= 0)
+         return(ret);
+
+     } else if(typ == 2) {
+       /*  @abbr{...}, @code{...}, @file{...}, @i{...}, @option{...}, @r{...},
+             @ref{...}, @samp{...},@var{...}, get mapped to ... .
+       */
+       rpt+= strlen(found);
+       ret= Mx1_rewrap(m, &rpt, &wpt, line_out, found, "", "", flag & 1);
+       if(ret <= 0)
+         return(ret);
+
+     } else if(typ == 3) {
+       /* @ref{...}, @xref{...} get mapped to empty text.
+       */
+       rpt+= strlen(found);
+       ret= Mx1_rewrap(m, &rpt, &wpt, line_out, found , "", "",
+                       (flag & 1) | 2);
+       if(ret <= 0)
+         return(ret);
+
+     } else if(strncmp(rpt, "@email{", 7) == 0 && !(flag & 1)) {
+       /* @email{...} gets mapped to <...> . */
+       rpt+= 7;
+       ret= Mx1_rewrap(m, &rpt, &wpt, line_out, "@email{", "<", ">", 0);
+       if(ret <= 0)
+         return(ret);
+
      } else if(strncmp(rpt, "@minus{}", 8) == 0) {
        /* @minus{} will become "-". */
        if((wpt - line_out) + 1 > 255)
          goto overflow;
        *(wpt++)= '-';
        rpt+= 7;
+
      } else if(strncmp(rpt, "@@", 2) == 0 ||
                strncmp(rpt, "@{", 2) == 0 ||
                strncmp(rpt, "@}", 2) == 0) {
        /* @@ , @{, @} will get stripped of their first @. */
+       if((wpt - line_out) + 1 > 255)
+         goto overflow;
        *(wpt++)= *(rpt + 1);
        rpt++;
-     }  
+
+     } else {
+       if((wpt - line_out) + 1 > 255)
+         goto overflow;
+       *(wpt++)= *(rpt);
+
+     }
+
    } else if(*rpt == '\\') {
      /* "\" becomes "\\" */
      if((wpt - line_out) + 2 > 255)
        goto overflow;
      *(wpt++)= '\\';
      *(wpt++)= '\\';
+
    } else if((wpt - line_out) + 1 > 255) {
 overflow:;
      Mx1_report_error(m, "Line length overflow while text substitution", 0);
      return(-1);
    } else
      *(wpt++)= *rpt;
+
  }
  *wpt= 0;
  return(1);
@@ -147,8 +300,8 @@ overflow:;
 */
 int Mx1_convert(struct Mx1 *m, char line_in[256], char line_out[256], int flag)
 {
- int l, num, keep= 0, ret, raw;
- char word[256], buf[256], *remainder;
+ int l, num, keep= 0, ret, raw, i, backslash_count;
+ char word[256], buf[256], *remainder, *wpt;
 
  m->count_in++;
  l= strlen(line_in);
@@ -233,7 +386,7 @@ int Mx1_convert(struct Mx1 *m, char line_in[256], char line_out[256], int flag)
    /* Substitute option text */
    raw= strlen(buf);
    strcat(buf, word);
-   ret= Mx1_substitute(m, buf, line_out, raw, 0);
+   ret= Mx1_substitute(m, buf, line_out, raw, strlen(buf), 0);
    if(ret <= 0)
      return(-1);
    strcpy(buf, line_out);
@@ -243,26 +396,28 @@ int Mx1_convert(struct Mx1 *m, char line_in[256], char line_out[256], int flag)
    strcat(buf, remainder);
 
    /* Substitute arguments text */
-   ret= Mx1_substitute(m, buf, line_out, raw, 0);
+   ret= Mx1_substitute(m, buf, line_out, raw, strlen(buf), 0);
    if(ret <= 0)
      return(-1);
    m->count_out++;
    return(1);
  }
 
- /* @strong{-...} gets mapped to \fB\-...\fR . */
  /* @strong{... } gets mapped to \fB...\fR . */
+ /* @command{... } gets mapped to \fB...\fR . */
  /* @minus{} will become "-". */
+ /* Mapped {...} content is subject to the rules except {...} mapping. */
  /* @@ , @{, @} will get stripped of their first @. */
  /* "\" becomes "\\" */
+
  if(line_in[0] != '@' ||
-    strncmp(line_in, "@strong{", 8) == 0 ||
+    Mx1_is_any_wrap(m, line_in, NULL, 0) > 0 ||
     strncmp(line_in, "@minus{}", 8) == 0 ||
     strncmp(line_in, "@@", 2) == 0 ||
     strncmp(line_in, "@{", 2) == 0 ||
-    strncmp(line_in, "@}", 2) == 0) {
+    strncmp(line_in, "@}", 2) == 0 ) {
    keep= 1;
-   ret= Mx1_substitute(m, line_in, line_out, 0, 0);
+   ret= Mx1_substitute(m, line_in, line_out, 0, strlen(line_in), 0);
    if(ret <= 0)
      return(-1);
  }
@@ -273,6 +428,26 @@ int Mx1_convert(struct Mx1 *m, char line_in[256], char line_out[256], int flag)
      return(0);
    strcpy(line_out, line_in);
  }
+
+ /* "-" which are not preceded by an uneven number of "\"  will get
+    prepended one "\".
+ */
+ l= strlen(line_out);
+ backslash_count= 0;
+ wpt= buf;
+ for(i= 0; i < l; i++) {
+   if(line_out[i] == '\\')
+     backslash_count++;
+   else if(line_out[i] == '-') {
+     if(backslash_count % 2 == 0)
+       *(wpt++)= '\\';
+     backslash_count= 0;
+   } else
+     backslash_count= 0;
+   *(wpt++)= line_out[i];
+ }
+ *wpt= 0;
+ strcpy(line_out, buf);
  m->count_out++;
  return(1);
 }
@@ -291,7 +466,7 @@ int main(int argc, char **argv)
 
  if(argc < 2) {
 usage:;
-   fprintf(stderr, "usage:  %s -auto|-filter|[-xorrisofs]\n", argv[0]);
+   fprintf(stderr, "usage:  %s -auto|-filter [-xorrisofs]\n", argv[0]);
    fprintf(stderr, "  -auto       xorriso/xorriso.texi -> xorriso/xorriso.1\n");
    fprintf(stderr, "  -filter     stdin                -> stdout\n");
    fprintf(stderr, "  -xorrisofs  process xorriso/xorrisofs.texi\n");
