@@ -36,18 +36,24 @@
 
 
 /* Option -abort_on */
-int Xorriso_option_abort_on(struct XorrisO *xorriso, char *severity, int flag)
+int Xorriso_option_abort_on(struct XorrisO *xorriso, char *in_severity,
+                            int flag)
 {
  int ret, sev;
+ char severity[20], *official;
 
+ Xorriso__to_upper(in_severity, severity, (int) sizeof(severity), 0);
  ret= Xorriso__text_to_sev(severity, &sev, 0);
  if(ret<=0) {
    sprintf(xorriso->info_text, "-abort_on: Not a known severity name : ");
-   Text_shellsafe(severity, xorriso->info_text, 1);
+   Text_shellsafe(in_severity, xorriso->info_text, 1);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
    return(ret);
  }
- if(Sfile_str(xorriso->abort_on_text,severity,0)<=0)
+ ret= Xorriso__sev_to_text(sev, &official, 0);
+ if(ret <= 0)
+   official= severity;
+ if(Sfile_str(xorriso->abort_on_text, official, 0) <= 0)
    return(-1);
  xorriso->abort_on_severity= sev;
  xorriso->abort_on_is_default= 0;
