@@ -101,8 +101,9 @@ int Xorriso_option_dev(struct XorrisO *xorriso, char *in_adr, int flag)
 }
 
 
-/* Option -devices */
-/* @return <=0 error , 1 success, 2 revoked by -reassure
+/* Option -devices , -device_links */
+/* @param flag bit0= perform -device_links rather than -devices
+   @return <=0 error , 1 success, 2 revoked by -reassure
 */
 int Xorriso_option_devices(struct XorrisO *xorriso, int flag)
 {
@@ -138,7 +139,7 @@ int Xorriso_option_devices(struct XorrisO *xorriso, int flag)
    }
    Xorriso_give_up_drive(xorriso, 3);
  }
- ret= Xorriso_show_devices(xorriso, 0);
+ ret= Xorriso_show_devices(xorriso, flag & 1);
  return(ret);
 }
 
@@ -1510,6 +1511,8 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -gid gid       Group id for the same purpose.",
 "",
 "  -devices       Show list of available optical drives and their addresses.",
+"  -device_links  Like devices, but showing link paths which are hopefully",
+"                 persistent over reboot on modern Linux systems.",
 "",
 "  -toc           Show media specific table of content (sessions).",
 "",
