@@ -447,7 +447,8 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
  int ret, was_dashed= 0, i, cmd_data_size= 2 * SfileadrL;
  char *cmd, *cmd_data= NULL;
  static char arg0_commands[][40]= {
-    "ban_stdio_write","close_filter_list","commit","devices","end",
+    "ban_stdio_write","close_filter_list","commit",
+    "device_links","devices","end",
     "for_backup", "help",
     "list_formats","list_speeds",
     "no_rc","print_size","pvd_info","pwd","pwdi","pwdx",
@@ -844,6 +845,9 @@ next_command:;
    (*idx)++;
    ret= Xorriso_option_dev(xorriso, arg1, 3);
 
+ } else if(strcmp(cmd,"device_links")==0) {
+   ret= Xorriso_option_devices(xorriso, 1);
+
  } else if(strcmp(cmd,"devices")==0) {
    ret= Xorriso_option_devices(xorriso, 0);
 
