@@ -110,8 +110,8 @@
    version of xorriso.
 */
 #define Xorriso_req_majoR  1
-#define Xorriso_req_minoR  0
-#define Xorriso_req_micrO  8
+#define Xorriso_req_minoR  1
+#define Xorriso_req_micrO  4
 
 
 static void yell_xorriso()
@@ -226,14 +226,20 @@ int main(int argc, char **argv)
 
  /* The prescan of arguments performs actions which have to happen before
     the normal processing of startup files and arguments.
+    Among them are -help and -prog_help which end the program without
+    yelling its name and version.
  */
  ret= Xorriso_prescan_args(xorriso,argc,argv,0);
  if(ret == 0)
    goto end_successfully;
+ /* Put out program name and version to stderr only if not done already now */
+ yell_xorriso();
+ if(ret < 0)
+   exit(5);
+ /* After having yelled xorriso, prescan again for unknown arguments */
+ ret= Xorriso_prescan_args(xorriso, argc, argv, 2);
  if(ret < 0)
    exit(5);
-
- yell_xorriso();
 
  /* The following command interpreters are allowed only after this
     initialization.
